/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#include <CStackApplet.h>

// Qt includes
#include <QApplication>
#include <QSettings>
#include <QSharedPointer>

// StackApplet includes
#include <CInboxItem.h>
#include <dialogs/CAboutDialog.h>
#include <dialogs/CAuthDialog.h>
#include <dialogs/CPreferencesDialog.h>

CStackApplet::CStackApplet()
    : m_have_access(false)
{
    // The API emits a signal when inbox items are retrieved from the API and
    // when an error occurs.
    connect(&m_api, SIGNAL(InboxItems(CInboxItemList)), SLOT(OnInboxItems(CInboxItemList)));
    connect(&m_api, SIGNAL(Error(QString)),    &m_icon, SLOT(DisplayError(QString)));

    // The auth dialog will either return an error or an access token
    connect(&m_auth, SIGNAL(Error(QString)), &m_icon, SLOT(DisplayError(QString)));
    connect(&m_auth, SIGNAL(AccessToken(QString)),    SLOT(OnAccessToken(QString)));

    // The tray icon sends events to us when the user clicks on items
    // in the context menu.
    connect(&m_icon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
                     SLOT(OnIconClicked(QSystemTrayIcon::ActivationReason)));
    connect(&m_icon, SIGNAL(Authorize()),   &m_auth,                  SLOT(Authorize()));
    connect(&m_icon, SIGNAL(About()),       SLOT(OnAbout()));
    connect(&m_icon, SIGNAL(Preferences()), SLOT(OnPreferences()));
    connect(&m_icon, SIGNAL(Quit()),        QApplication::instance(), SLOT(quit()));

    // The popup will tell us when the user wants to mark all as read
    connect(&m_popup, SIGNAL(MarkAllRead()), SLOT(OnMarkAllRead()));
    connect(&m_popup, SIGNAL(MarkAllRead()), &m_icon, SLOT(ShowNoMessageIcon()));

    // The timer fires off an event during each 'tick' - and we do two things
    // when such an event occurs - display the loading message in the popup and
    // have the API update. Later we will also display a refreshing icon.
    connect(&m_timer, SIGNAL(timeout()), &m_api,   SLOT(Update()));
    connect(&m_timer, SIGNAL(timeout()), &m_popup, SLOT(DisplayLoading()));

    // Set the timer's interval
    m_timer.setSingleShot(true);
    m_timer.setInterval(120000);

    // Load some settings
    QSettings settings;

    QString access_token = settings.value("api/access_token",   "").toString();
    m_last_link_seen     = settings.value("api/last_link_seen", "").toString();
    m_last_link_read     = settings.value("api/last_link_read", "").toString();

    // If a valid access token already exists, then start issuing requests
    if(!access_token.isEmpty())
        OnAccessToken(access_token);
}

CStackApplet::~CStackApplet()
{
    // Under ideal circumstances, the destructor would save all of the
    // settings and write them to disk, but... when certain operating
    // systems are shut down, they aren't the best at alerting running
    // applications that they should be closing - so sometimes the rug
    // gets pulled out from the application. Therefore StackApplet saves
    // settings as it runs - instead of all at the end.
}

void CStackApplet::OnAbout()
{
    CAboutDialog().exec();
}

void CStackApplet::OnAccessToken(QString access_token)
{
    // Immediately store the new access token
    QSettings().setValue("api/access_token", access_token);

    m_have_access = true;

    // Immediately issue an update
    m_api.SetAccessToken(access_token);
    m_api.Update();
}

void CStackApplet::OnIconClicked(QSystemTrayIcon::ActivationReason reason)
{
    // Only display the menu if the icon was clicked with the left button
    if(reason == QSystemTrayIcon::Trigger)
    {
        // Position it and show it
        m_popup.Position(m_icon.geometry());
        m_popup.show();
    }
}

void CStackApplet::OnInboxItems(CInboxItemList items)
{
    // Clear the old list of unread items
    m_unread.clear();

    // Loop through the items noting the last point at which
    // we recognize something that we already have seen. Then
    // continue looking for the last item the user has marked
    // as read so that we can determine whether or not we need
    // to alert the user that there are new items.
    bool found_last_seen = false;
    int num_unseen = 0;

    foreach(QSharedPointer<CInboxItem> item, items)
    {
        // Check to see if this item is the last one we've seen
        // Yes, believe it or not the extra parenthesis actually
        // avoid a warning with GCC / Mingw32 :P
        if((found_last_seen = (!found_last_seen && item->link() == m_last_link_seen)))
            num_unseen = m_unread.size();

        // If the item is read or marked as read, we're done
        if(item->link() == m_last_link_read || !item->isUnread())
            break;

        // ...otherwise we append it to the unread list
        m_unread.append(item);
    }

    // Now generate the HTML for the popup - this will be the list of
    // unread items if there are any otherwise it will be blank
    if(m_unread.size())
    {
        // Generate the HTML for the list of unread items
        QStringList inbox_items;

        foreach(QSharedPointer<CInboxItem> item, m_unread)
            inbox_items.append(tr("<li><a href='%1'>%2</a></li>")
                               .arg(item->link())
                               .arg(item->title()));

        // Join the items together and set the contents of the popup
        m_popup.SetContents(tr("<ul>%1</ul>").arg(inbox_items.join("")));

        // Also activate the icon indicating there are new items
        m_icon.ShowMessageIcon();

        // Display a notification IF there were any messages that we haven't seen
        // and also be sure to store the link to the last item we've seen (the first)
        if(num_unseen)
        {
            m_icon.DisplayNotification(tr("%1 New Item(s)").arg(num_unseen),
                                       tr("There are %1 new item(s) in your inbox.").arg(num_unseen));

            // Store the last one seen
            m_last_link_seen = items.at(0)->link();
            QSettings().setValue("api/last_link_seen", m_last_link_seen);
        }
    }
    else
    {
        // Set the contents of the popup and the icon
        m_popup.SetContents(tr("<span>No unread items.</span>"));
        m_icon.ShowNoMessageIcon();
    }

    // Set the timer for next time
    m_timer.start();
}

void CStackApplet::OnMarkAllRead()
{
    // We perform two important tasks here - we record
    // the topmost (first) item in the unread list and
    // then clear the unread list - so there is basically
    // nothing unread until a new item comes in.
    if(m_unread.size())
    {
        m_last_link_read = m_unread.at(0)->link();
        QSettings().setValue("api/last_link_read", m_last_link_read);
    }

    m_unread.clear();
    m_popup.SetContents(tr("<span>No unread items.</span>"));
}

void CStackApplet::OnPreferences()
{
    CPreferencesDialog().exec();
}
