/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#include <dialogs/CInfoPopup.h>

// Qt includes
#include <QApplication>
#include <QDesktopServices>
#include <QDesktopWidget>
#include <QEvent>
#include <QWebFrame>
#include <QWebElement>

// Initialize the static members
const int CInfoPopup::PopupWidth   = 400;
const int CInfoPopup::PopupHeight  = 140;
const int CInfoPopup::PopupHorizontalPadding = 30;
const int CInfoPopup::PopupVerticalPadding   = 20;

CInfoPopup::CInfoPopup()
{
    // Initialize the window
    setWindowTitle("StackApplet Popup Window");  // Just in case some window
                                                 // manager decides to be funny here
    setWindowFlags(Qt::Popup |
                   Qt::WindowStaysOnTopHint |
                   Qt::FramelessWindowHint);
    setWindowOpacity(0.8);

    // Set the contents
    setUrl(QUrl("qrc:/html/popup.html"));

    // Install the event filter so we can hide the popup when focus is lost
    // and set a handler for links
    installEventFilter(this);

    page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
    connect(this, SIGNAL(linkClicked(QUrl)), SLOT(OnLinkClicked(QUrl)));

    settings()->setAttribute(QWebSettings::DeveloperExtrasEnabled, true);
}

void CInfoPopup::Position(QRect icon_rect)
{
    // Begin by retrieving the dimensions of the primary display.
    QDesktopWidget * desktop = QApplication::desktop();
    QRect desk_dim   = desktop->screenGeometry(desktop->primaryScreen());
    QRect desk_avail = desktop->availableGeometry(desktop->primaryScreen());

    // Now, determine which corner we should stick the popup in by
    // figuring out what corner (approximately) the tray icon is in.

    // Some platform specific notes here - on Windows, the primary display
    // always has the start menu, so we can be assured that the system tray
    // is displayed on the primary display. We need to merely determine which
    // corner it is in.

    int xpos = 0,  // the eventual location where
        ypos = 0;  // the popup will be placed

    // There really are only three reasonable posibilities for what corner the
    // tray is in since it will never be in the upper-left
    int half_width  = desk_dim.width() / 2;
    int half_height = desk_dim.height() / 2;

    // Check for the left half of screen
    if(icon_rect.left() < half_width)
    {
        // It's in the lower-left, so then display it xx px from each corner
        xpos = desk_avail.left() + PopupHorizontalPadding;
        ypos = desk_avail.bottom() - PopupVerticalPadding - PopupHeight;
    }
    else
    {
        // It could be either the top right, or bottom right for the remaining 3
        // possible orientations
        if(icon_rect.top() < half_height)
        {
            // The top right orientation
            xpos = desk_avail.right() - PopupHorizontalPadding - PopupWidth;
            ypos = desk_avail.top() + PopupVerticalPadding;
        }
        else
        {
            // Both of the bottom right orientations
            xpos = desk_avail.right() - PopupHorizontalPadding - PopupWidth;
            ypos = desk_avail.bottom() - PopupVerticalPadding - PopupHeight;
        }
    }

    // Now move the window to such location
    setGeometry(xpos, ypos, PopupWidth, PopupHeight);
}

void CInfoPopup::SetContents(QString contents)
{
    // Dig up the #content div and insert the specified contents
    QWebElement element = page()->mainFrame()->findFirstElement("#data");

    // If we found the element, set its contents
    if(!element.isNull())
        element.setInnerXml(contents);
}

bool CInfoPopup::eventFilter(QObject * object, QEvent * event)
{
    if(event->type() == QEvent::WindowDeactivate)
    {
        hide();
        return TRUE;
    }

    return QObject::eventFilter(object, event);
}

void CInfoPopup::DisplayLoading()
{
    SetContents(tr("<span>Refreshing inbox...</span>"));
}

void CInfoPopup::OnLinkClicked(QUrl url)
{
    // If it is an external link, launch the user's default browser.
    // If it is not, then take the appropriate action.

    if(url.scheme() == "http" || url.scheme() == "https")
        QDesktopServices::openUrl(url);
    else if(url.path() == "markread")
        emit MarkAllRead();
}
